
package org.mule.module.magento.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.magento.processors.CreateShoppingCartOrderMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class CreateShoppingCartOrderDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public CreateShoppingCartOrderDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(CreateShoppingCartOrderMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("quoteId")!= null)&&(!StringUtils.isBlank(element.getAttribute("quoteId")))) {
            builder.addPropertyValue("quoteId", element.getAttribute("quoteId"));
        }
        if ((element.getAttribute("storeId")!= null)&&(!StringUtils.isBlank(element.getAttribute("storeId")))) {
            builder.addPropertyValue("storeId", element.getAttribute("storeId"));
        }
        Element licensesListElement = null;
        licensesListElement = DomUtils.getChildElementByTagName(element, "licenses");
        List<Element> licensesListChilds = null;
        if (licensesListElement!= null) {
            String licensesRef = licensesListElement.getAttribute("ref");
            if ((licensesRef!= null)&&(!StringUtils.isBlank(licensesRef))) {
                if ((!licensesRef.startsWith(patternInfo.getPrefix()))&&(!licensesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("licenses", new RuntimeBeanReference(licensesRef));
                } else {
                    builder.addPropertyValue("licenses", licensesRef);
                }
            } else {
                ManagedList licenses = new ManagedList();
                licensesListChilds = DomUtils.getChildElementsByTagName(licensesListElement, "license");
                if (licensesListChilds!= null) {
                    for (Element licensesChild: licensesListChilds) {
                        String valueRef = licensesChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            if ((!valueRef.startsWith(patternInfo.getPrefix()))&&(!valueRef.endsWith(patternInfo.getSuffix()))) {
                                licenses.add(new RuntimeBeanReference(valueRef));
                            } else {
                                licenses.add(valueRef);
                            }
                        } else {
                            licenses.add(licensesChild.getTextContent());
                        }
                    }
                }
                builder.addPropertyValue("licenses", licenses);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
