
package org.mule.module.magento.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.magento.processors.SetShoppingCartCustomerMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class SetShoppingCartCustomerDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public SetShoppingCartCustomerDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(SetShoppingCartCustomerMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("quoteId")!= null)&&(!StringUtils.isBlank(element.getAttribute("quoteId")))) {
            builder.addPropertyValue("quoteId", element.getAttribute("quoteId"));
        }
        Element customerListElement = null;
        customerListElement = DomUtils.getChildElementByTagName(element, "customer");
        List<Element> customerMapChilds = null;
        if (customerListElement!= null) {
            String customerRef = customerListElement.getAttribute("ref");
            if ((customerRef!= null)&&(!StringUtils.isBlank(customerRef))) {
                if ((!customerRef.startsWith(patternInfo.getPrefix()))&&(!customerRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("customer", new RuntimeBeanReference(customerRef));
                } else {
                    builder.addPropertyValue("customer", customerRef);
                }
            } else {
                ManagedMap customer = new ManagedMap();
                customerMapChilds = DomUtils.getChildElementsByTagName(customerListElement, "customer");
                if (customerMapChilds!= null) {
                    if (customerMapChilds.size() == 0) {
                        customerMapChilds = DomUtils.getChildElements(customerListElement);
                    }
                    for (Element customerChild: customerMapChilds) {
                        String customerValueRef = customerChild.getAttribute("value-ref");
                        String customerKeyRef = customerChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((customerValueRef!= null)&&(!StringUtils.isBlank(customerValueRef))) {
                            if ((!customerValueRef.startsWith(patternInfo.getPrefix()))&&(!customerValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(customerValueRef);
                            } else {
                                valueObject = customerValueRef;
                            }
                        } else {
                            valueObject = customerChild.getTextContent();
                        }
                        if ((customerKeyRef!= null)&&(!StringUtils.isBlank(customerKeyRef))) {
                            keyObject = new RuntimeBeanReference(customerKeyRef);
                        } else {
                            keyObject = customerChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = customerChild.getTagName();
                        }
                        customer.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("customer", customer);
            }
        }
        if ((element.getAttribute("storeId")!= null)&&(!StringUtils.isBlank(element.getAttribute("storeId")))) {
            builder.addPropertyValue("storeId", element.getAttribute("storeId"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
